#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_file.interfaces import IFileInfo

__docformat__ = 'restructuredtext'


# import standard library
import json
import zipfile
from io import BytesIO

# import interfaces
from pyams_content.component.gallery.interfaces import IGallery, IGalleryRenderer, IGalleryFile
from pyams_content.component.gallery.zmi.interfaces import IGalleryImageAddFields, IGalleryImagesView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IWidgetsPrefixViewletsManager
from pyams_i18n.interfaces import II18n
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION

# import packages
from pyams_content.shared.common.zmi import WfSharedContentPermissionMixin
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_template.template import template_config, get_view_template
from pyams_utils.adapter import adapter_config, ContextRequestAdapter
from pyams_viewlet.viewlet import viewlet_config, Viewlet
from pyams_zmi.form import AdminDialogEditForm, AdminDialogDisplayForm
from pyramid.exceptions import NotFound
from pyramid.renderers import render_to_response
from pyramid.response import Response
from pyramid.view import view_config
from z3c.form import field
from zope.interface import implementer, Interface

from pyams_content import _


#
# Gallery properties
#

@pagelet_config(name='properties.html', context=IGallery, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class GalleryPropertiesEditForm(AdminDialogEditForm):
    """Gallery properties edit form"""

    legend = _("Update gallery properties")
    icon_css_class = 'fa fa-fw fa-picture-o'

    fields = field.Fields(IGallery).omit('__parent__', '__file__')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(GalleryPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IGallery, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class GalleryPropertiesAJAXEditForm(AJAXEditForm, GalleryPropertiesEditForm):
    """Gallery properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if 'title' in changes.get(IGallery, ()):
            return {'status': 'reload',
                    'location': '#external-files.html'}
        else:
            return super(GalleryPropertiesAJAXEditForm, self).get_ajax_output(changes)


#
# Gallery contents
#

@pagelet_config(name='contents.html', context=IGallery, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@implementer(IGalleryImagesView)
class GalleryContentForm(AdminDialogDisplayForm):
    """Gallery contents form"""

    legend = _("Update gallery contents")
    dialog_class = 'modal-max'

    fields = field.Fields(Interface)
    show_widget_title = True


@viewlet_config(name='gallery-images', context=IGallery, view=IGalleryImagesView, manager=IWidgetsPrefixViewletsManager)
@template_config(template='templates/gallery-images.pt', layer=IPyAMSLayer)
@implementer(IGalleryImagesView)
class GalleryImagesViewlet(Viewlet):
    """Gallery images viewlet"""

    def get_title(self, image):
        return II18n(image).query_attribute('title', request=self.request)


@view_config(name='get-gallery-images.html', context=IGallery, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION)
@implementer(IGalleryImagesView)
class GalleryImagesView(WfSharedContentPermissionMixin):
    """Gallery images view"""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        return render_to_response('templates/gallery-images.pt', {'view': self}, request=self.request)

    def get_title(self, image):
        return II18n(image).query_attribute('title', request=self.request)


@view_config(name='set-images-order.json', context=IGallery, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_images_order(request):
    """Set gallery images order"""
    images_names = json.loads(request.params.get('images'))
    request.context.updateOrder(images_names)
    return {'status': 'success'}


@view_config(name='set-image-visibility.json', context=IGallery, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_image_visibility(request):
    """Set gallery image visibility"""
    gallery = IGallery(request.context)
    image = gallery.get(str(request.params.get('object_name')))
    if image is None:
        raise NotFound()
    image.visible = not image.visible
    return {'visible': image.visible}


#
# Gallery renderers
#

class BaseGalleryRenderer(ContextRequestAdapter):
    """Base gallery renderer"""

    def update(self):
        pass

    render = get_view_template()


@adapter_config(name='default', context=(IGallery, IPyAMSLayer), provides=IGalleryRenderer)
@template_config(template='templates/renderer-default.pt', layer=IPyAMSLayer)
class DefaultGalleryRenderer(BaseGalleryRenderer):
    """Default gallery renderer"""

    label = _("Default gallery renderer")
    weight = 1


#
# Gallery images downloader
#

@view_config(name='get-images.zip', context=IGallery, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION)
def get_images_archive(request):
    """Get all gallery images as ZIP file"""
    zip_data = BytesIO()
    zip_file = zipfile.ZipFile(zip_data, mode='w')
    for image in IGallery(request.context).values():
        zip_file.writestr(IFileInfo(image.data).filename, image.data.data)
    zip_file.close()
    zip_data.seek(0)
    response = Response(content_type='application/zip',
                        content_disposition='attachment; filename="gallery.zip"')
    response.body_file = zip_data
    return response
